  #define MAX_KEY  256
  #define BUF_SIZE 1024

  typedef struct {
	unsigned short int     type;
	unsigned short int     version;
	long     root;
	long     next_page;
	unsigned short int     item_size;
	unsigned short int     key_size;
	unsigned short int     key_dec;
	unsigned short int     max_item;
	unsigned short int     half_page;
	char     key_expr[MAX_KEY];
	char     unique;
  } NTX_HEADER;

  typedef struct
  {
	long page;
	long rec_no;
	char key[1];
  } NTX_ITEM;

  typedef struct
  {
	unsigned short item_count;
	unsigned short item_offset[1];
  } NTX_BUFFER;

  #include "stdio.h"
  #include "stdlib.h"
  #include "fcntl.h"
  #include "io.h"

  NTX_HEADER ntx_header;
  int        ntx_handle;

  void view_page(long page_offset);


  void main(int argc, char ** argv)
  {
	if( argc != 2 ) {
	    printf( "Use: ntxhead arquivo.ntx\n" );
	    exit(1);
	}

	ntx_handle = open(argv[1], 0x8000);
	if( ntx_handle == -1 ) {
	    printf( "Open error: %s\n", argv[1] );
	    exit(1);
	}

	/* Reads header */
	if( read(ntx_handle, &ntx_header, sizeof(NTX_HEADER))
	     != sizeof(NTX_HEADER) ) {
	    printf( "Error reading header of %s\n", argv[1] );
	    exit(1);
	}

	/* Prints header information */
	printf( "***** NTX header *****\n" );
	printf( "Signature %xh\n", ntx_header.type );
	printf( "Version %xh\n", ntx_header.version );
	printf( "Root %lxh\n", ntx_header.root );
	printf( "Next page %xh\n", ntx_header.next_page );
	printf( "Item size %xh\n", ntx_header.item_size );
	printf( "Key size %xh\n", ntx_header.key_size );
	printf( "Key decimals %xh\n", ntx_header.key_dec );
	printf( "Max item %xh\n", ntx_header.half_page );
	printf( "Key expr: '%s'\n", ntx_header.key_expr );
	printf( "Unique: %d\n", ntx_header.unique );

	view_page(ntx_header.root);

	close(ntx_handle);
    }


  void view_page(long page_offset)
  {
	   char       * page;
	   NTX_BUFFER * buffer;
	   NTX_ITEM   * item;
	   unsigned char   i;

	/* Allocates page */
	page = malloc(BUF_SIZE);
	buffer = (NTX_BUFFER *) page;

  printf( "view_page\n" );

	if( page == NULL ) {
	    printf( "Error allocating memory.\n" );
	    exit(1);
	}

	/* Moves to the given page offset within the file */
	lseek(ntx_handle, page_offset, 0);

	/* Reads page */
	if( read(ntx_handle, buffer, BUF_SIZE) != BUF_SIZE ) {
	    printf( "Error reading page %lxh\n", page_offset );
	    exit(1);
	}

  printf( "items: %i\n", buffer->item_count );

	/* Traverses the vector, item by item */
	for( i = 0; i < buffer->item_count; i ++ ) 
	{
	   item = (NTX_ITEM *) (page + buffer->item_offset[i]);
	   if( item->page != 0 )
		    view_page(item->page);
	   printf( "%7i recno: %3i Key: %s\n", i, item->rec_no, &item->key);
	}

	/* Handles last pointer */
	item = (NTX_ITEM *) (page + buffer->item_offset[buffer->item_count]);
	if( item->page )
	    view_page(item->page);

	/* Deallocates page */
	free(page);
    }
